@@ -1,23 +1,30 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * e-table-model.c: a simple table model implementation that uses function
- * pointers to simplify the creation of new, exotic and colorful tables in
- * no time.
+/* 
+ * e-table-simple.c
+ * Copyright 2000, 2001, Ximian, Inc.
  *
- * Author:
- *   Miguel de Icaza (miguel@gnu.org)
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
+ *   Miguel de Icaza <miguel.ximian.com>
  *
- * (C) 1999, 2000 Helix Code, Inc.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
 
 #include <config.h>
 #include "e-table-simple.h"
 
-enum {
-	ARG_0,
-	ARG_APPEND_ROW,
-};
-
 #define PARENT_TYPE e_table_model_get_type ()
 
 static int
@@ -42,6 +49,15 @@ simple_row_count (ETableModel *etm)
 		return 0;
 }
 
+static void
+simple_append_row (ETableModel *etm, ETableModel *source, int row)
+{
+	ETableSimple *simple = E_TABLE_SIMPLE(etm);
+	
+	if (simple->append_row)
+		simple->append_row (etm, source, row, simple->data);
+}
+
 static void *
 simple_value_at (ETableModel *etm, int col, int row)
 {
@@ -73,6 +89,28 @@ simple_is_cell_editable (ETableModel *etm, int col, int row)
 		return FALSE;
 }
 
+static gboolean
+simple_has_save_id (ETableModel *etm)
+{
+	ETableSimple *simple = E_TABLE_SIMPLE(etm);
+
+	if (simple->has_save_id)
+		return simple->has_save_id (etm, simple->data);
+	else
+		return FALSE;
+}
+
+static char *
+simple_get_save_id (ETableModel *etm, int row)
+{
+	ETableSimple *simple = E_TABLE_SIMPLE(etm);
+
+	if (simple->get_save_id)
+		return simple->get_save_id (etm, row, simple->data);
+	else
+		return NULL;
+}
+
 /* The default for simple_duplicate_value is to return the raw value. */
 static void *
 simple_duplicate_value (ETableModel *etm, int col, const void *value)
@@ -127,63 +165,27 @@ simple_value_to_string (ETableModel *etm, int col, const void *value)
 		return g_strdup ("");
 }
 
-static void
-simple_append_row (ETableModel *etm, ETableModel *source, int row)
-{
-	ETableSimple *simple = E_TABLE_SIMPLE(etm);
-	
-	if (simple->append_row)
-		simple->append_row (etm, source, row, simple->data);
-}
-
-static void
-simple_get_arg (GtkObject *o, GtkArg *arg, guint arg_id)
-{
-	ETableSimple *simple = E_TABLE_SIMPLE (o);
-
-	switch (arg_id){
-	case ARG_APPEND_ROW:
-		GTK_VALUE_POINTER(*arg) = simple->append_row;
-		break;
-	}
-}
-
-static void
-simple_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
-{
-	ETableSimple *simple = E_TABLE_SIMPLE (o);
-	
-	switch (arg_id){
-	case ARG_APPEND_ROW:
-		simple->append_row = GTK_VALUE_POINTER(*arg);
-		break;
-	default:
-		arg->type = GTK_TYPE_INVALID;
-	}
-}
-
 static void
 e_table_simple_class_init (GtkObjectClass *object_class)
 {
 	ETableModelClass *model_class = (ETableModelClass *) object_class;
 
-	object_class->set_arg = simple_set_arg;
-	object_class->get_arg = simple_get_arg;
+	model_class->column_count      = simple_column_count;
+	model_class->row_count         = simple_row_count;
+	model_class->append_row        = simple_append_row;
 
-	model_class->column_count = simple_column_count;
-	model_class->row_count = simple_row_count;
-	model_class->value_at = simple_value_at;
-	model_class->set_value_at = simple_set_value_at;
-	model_class->is_cell_editable = simple_is_cell_editable;
-	model_class->duplicate_value = simple_duplicate_value;
-	model_class->free_value = simple_free_value;
-	model_class->initialize_value = simple_initialize_value;
-	model_class->value_is_empty = simple_value_is_empty;
-	model_class->value_to_string = simple_value_to_string;
-	model_class->append_row = simple_append_row;
+	model_class->value_at          = simple_value_at;
+	model_class->set_value_at      = simple_set_value_at;
+	model_class->is_cell_editable  = simple_is_cell_editable;
 
-	gtk_object_add_arg_type ("ETableSimple::append_row", GTK_TYPE_POINTER,
-				 GTK_ARG_READWRITE, ARG_APPEND_ROW);
+	model_class->has_save_id       = simple_has_save_id;
+	model_class->get_save_id       = simple_get_save_id;
+
+	model_class->duplicate_value   = simple_duplicate_value;
+	model_class->free_value        = simple_free_value;
+	model_class->initialize_value  = simple_initialize_value;
+	model_class->value_is_empty    = simple_value_is_empty;
+	model_class->value_to_string   = simple_value_to_string;
 }
 
 GtkType
@@ -237,33 +239,45 @@ e_table_simple_get_type (void)
  * object).
  */
 ETableModel *
-e_table_simple_new (ETableSimpleColumnCountFn col_count,
-		    ETableSimpleRowCountFn row_count,
-		    ETableSimpleValueAtFn value_at,
-		    ETableSimpleSetValueAtFn set_value_at,
-		    ETableSimpleIsCellEditableFn is_cell_editable,
-		    ETableSimpleDuplicateValueFn duplicate_value,
-		    ETableSimpleFreeValueFn free_value,
-		    ETableSimpleInitializeValueFn initialize_value,
-		    ETableSimpleValueIsEmptyFn value_is_empty,
-		    ETableSimpleValueToStringFn value_to_string,
-		    void *data)
+e_table_simple_new  (ETableSimpleColumnCountFn      col_count,
+		     ETableSimpleRowCountFn         row_count,
+		     ETableSimpleAppendRowFn        append_row,
+
+		     ETableSimpleValueAtFn          value_at,
+		     ETableSimpleSetValueAtFn       set_value_at,
+		     ETableSimpleIsCellEditableFn   is_cell_editable,
+
+		     ETableSimpleHasSaveIdFn        has_save_id,
+		     ETableSimpleGetSaveIdFn        get_save_id,
+
+		     ETableSimpleDuplicateValueFn   duplicate_value,
+		     ETableSimpleFreeValueFn        free_value,
+		     ETableSimpleInitializeValueFn  initialize_value,
+		     ETableSimpleValueIsEmptyFn     value_is_empty,
+		     ETableSimpleValueToStringFn    value_to_string,
+		     void                          *data)
 {
 	ETableSimple *et;
 
-	et = gtk_type_new (e_table_simple_get_type ());
+	et                    = gtk_type_new (e_table_simple_get_type ());
+
+	et->col_count         = col_count;
+	et->row_count         = row_count;
+	et->append_row        = append_row;
+
+	et->value_at          = value_at;
+	et->set_value_at      = set_value_at;
+	et->is_cell_editable  = is_cell_editable;
+
+	et->has_save_id       = has_save_id;
+	et->get_save_id       = get_save_id;
+
+	et->duplicate_value   = duplicate_value;
+	et->free_value        = free_value;
+	et->initialize_value  = initialize_value;
+	et->value_is_empty    = value_is_empty;
+	et->value_to_string   = value_to_string;
+	et->data              = data;
 
-	et->col_count = col_count;
-	et->row_count = row_count;
-	et->value_at = value_at;
-	et->set_value_at = set_value_at;
-	et->is_cell_editable = is_cell_editable;
-	et->duplicate_value = duplicate_value;
-	et->free_value = free_value;
-	et->initialize_value = initialize_value;
-	et->value_is_empty = value_is_empty;
-	et->value_to_string = value_to_string;
-	et->data = data;
-	
 	return (ETableModel *) et;
- }
+}
